@@ -1,4 +1,5 @@
-//! Crds Gossip
+//! Crds Gossip.
+//!
 //! This module ties together Crds and the push and pull gossip overlays.  The interface is
 //! designed to run with a simulator or over a UDP network connection with messages up to a
 //! packet::PACKET_DATA_SIZE size.
@@ -15,7 +16,6 @@ use {
         duplicate_shred::{self, DuplicateShredIndex, LeaderScheduleFn, MAX_DUPLICATE_SHREDS},
         ping_pong::PingCache,
     },
-    itertools::Itertools,
     rayon::ThreadPool,
     solana_ledger::shred::Shred,
     solana_sdk::{
@@ -28,42 +28,48 @@ use {
     std::{
         collections::{HashMap, HashSet},
         net::SocketAddr,
-        sync::Mutex,
+        sync::{Mutex, RwLock},
         time::Duration,
     },
 };
 
 #[derive(Default)]
 pub struct CrdsGossip {
-    pub crds: Crds,
+    pub crds: RwLock<Crds>,
     pub push: CrdsGossipPush,
     pub pull: CrdsGossipPull,
 }
 
 impl CrdsGossip {
-    /// process a push message to the network
+    /// Process a push message to the network.
+    ///
     /// Returns unique origins' pubkeys of upserted values.
     pub fn process_push_message(
-        &mut self,
+        &self,
         from: &Pubkey,
         values: Vec<CrdsValue>,
         now: u64,
-    ) -> HashSet<Pubkey> {
-        values
+    ) -> (usize, HashSet<Pubkey>) {
+        let results = self
+            .push
+            .process_push_message(&self.crds, from, values, now);
+        let mut success_count = 0;
+        let successfully_inserted_origin_set: HashSet<Pubkey> = results
             .into_iter()
-            .filter_map(|val| {
-                let origin = val.pubkey();
-                self.push
-                    .process_push_message(&mut self.crds, from, val, now)
-                    .ok()?;
-                Some(origin)
+            .filter_map(|result| {
+                if result.is_ok() {
+                    success_count += 1;
+                }
+                Result::ok(result)
             })
-            .collect()
+            .collect();
+
+        (success_count, successfully_inserted_origin_set)
     }
 
-    /// remove redundant paths in the network
+    /// Remove redundant paths in the network.
     pub fn prune_received_cache<I>(
-        &mut self,
+        &self,
         self_pubkey: &Pubkey,
         origins: I, // Unique pubkeys of crds values' owners.
         stakes: &HashMap<Pubkey, u64>,
@@ -71,30 +77,26 @@ impl CrdsGossip {
     where
         I: IntoIterator<Item = Pubkey>,
     {
-        origins
-            .into_iter()
-            .flat_map(|origin| {
-                self.push
-                    .prune_received_cache(self_pubkey, &origin, stakes)
-                    .into_iter()
-                    .zip(std::iter::repeat(origin))
-            })
-            .into_group_map()
+        self.push
+            .prune_received_cache_many(self_pubkey, origins, stakes)
     }
 
     pub fn new_push_messages(
-        &mut self,
+        &self,
         pending_push_messages: Vec<CrdsValue>,
         now: u64,
     ) -> HashMap<Pubkey, Vec<CrdsValue>> {
-        for entry in pending_push_messages {
-            let _ = self.crds.insert(entry, now, GossipRoute::LocalMessage);
+        {
+            let mut crds = self.crds.write().unwrap();
+            for entry in pending_push_messages {
+                let _ = crds.insert(entry, now, GossipRoute::LocalMessage);
+            }
         }
         self.push.new_push_messages(&self.crds, now)
     }
 
     pub(crate) fn push_duplicate_shred(
-        &mut self,
+        &self,
         keypair: &Keypair,
         shred: &Shred,
         other_payload: &[u8],
@@ -105,8 +107,8 @@ impl CrdsGossip {
         let pubkey = keypair.pubkey();
         // Skip if there are already records of duplicate shreds for this slot.
         let shred_slot = shred.slot();
-        if self
-            .crds
+        let mut crds = self.crds.write().unwrap();
+        if crds
             .get_records(&pubkey)
             .any(|value| match &value.value.data {
                 CrdsData::DuplicateShred(_, value) => value.slot == shred_slot,
@@ -125,8 +127,7 @@ impl CrdsGossip {
         )?;
         // Find the index of oldest duplicate shred.
         let mut num_dup_shreds = 0;
-        let offset = self
-            .crds
+        let offset = crds
             .get_records(&pubkey)
             .filter_map(|value| match &value.value.data {
                 CrdsData::DuplicateShred(ix, value) => {
@@ -150,14 +151,14 @@ impl CrdsGossip {
         });
         let now = timestamp();
         for entry in entries {
-            if let Err(err) = self.crds.insert(entry, now, GossipRoute::LocalMessage) {
+            if let Err(err) = crds.insert(entry, now, GossipRoute::LocalMessage) {
                 error!("push_duplicate_shred faild: {:?}", err);
             }
         }
         Ok(())
     }
 
-    /// add the `from` to the peer's filter of nodes
+    /// Add the `from` to the peer's filter of nodes.
     pub fn process_prune_msg(
         &self,
         self_pubkey: &Pubkey,
@@ -179,29 +180,29 @@ impl CrdsGossip {
         }
     }
 
-    /// refresh the push active set
-    /// * ratio - number of actives to rotate
+    /// Refresh the push active set.
     pub fn refresh_push_active_set(
-        &mut self,
+        &self,
         self_pubkey: &Pubkey,
         self_shred_version: u16,
         stakes: &HashMap<Pubkey, u64>,
         gossip_validators: Option<&HashSet<Pubkey>>,
         socket_addr_space: &SocketAddrSpace,
     ) {
+        let network_size = self.crds.read().unwrap().num_nodes();
         self.push.refresh_push_active_set(
             &self.crds,
             stakes,
             gossip_validators,
             self_pubkey,
             self_shred_version,
-            self.crds.num_nodes(),
+            network_size,
             CRDS_GOSSIP_NUM_ACTIVE,
             socket_addr_space,
         )
     }
 
-    /// generate a random request
+    /// Generate a random request.
     #[allow(clippy::too_many_arguments)]
     pub fn new_pull_request(
         &self,
@@ -231,19 +232,20 @@ impl CrdsGossip {
         )
     }
 
-    /// time when a request to `from` was initiated
+    /// Time when a request to `from` was initiated.
+    ///
     /// This is used for weighted random selection during `new_pull_request`
     /// It's important to use the local nodes request creation time as the weight
     /// instead of the response received time otherwise failed nodes will increase their weight.
-    pub fn mark_pull_request_creation_time(&mut self, from: Pubkey, now: u64) {
+    pub fn mark_pull_request_creation_time(&self, from: Pubkey, now: u64) {
         self.pull.mark_pull_request_creation_time(from, now)
     }
-    /// process a pull request and create a response
-    pub fn process_pull_requests<I>(&mut self, callers: I, now: u64)
+    /// Process a pull request and create a response.
+    pub fn process_pull_requests<I>(&self, callers: I, now: u64)
     where
         I: IntoIterator<Item = CrdsValue>,
     {
-        CrdsGossipPull::process_pull_requests(&mut self.crds, callers, now);
+        CrdsGossipPull::process_pull_requests(&self.crds, callers, now);
     }
 
     pub fn generate_pull_responses(
@@ -277,9 +279,9 @@ impl CrdsGossip {
             .filter_pull_responses(&self.crds, timeouts, response, now, process_pull_stats)
     }
 
-    /// process a pull response
+    /// Process a pull response.
     pub fn process_pull_responses(
-        &mut self,
+        &self,
         from: &Pubkey,
         responses: Vec<CrdsValue>,
         responses_expired_timeout: Vec<CrdsValue>,
@@ -288,7 +290,7 @@ impl CrdsGossip {
         process_pull_stats: &mut ProcessPullStats,
     ) {
         self.pull.process_pull_responses(
-            &mut self.crds,
+            &self.crds,
             from,
             responses,
             responses_expired_timeout,
@@ -308,7 +310,7 @@ impl CrdsGossip {
     }
 
     pub fn purge(
-        &mut self,
+        &self,
         self_pubkey: &Pubkey,
         thread_pool: &ThreadPool,
         now: u64,
@@ -323,9 +325,11 @@ impl CrdsGossip {
             //sanity check
             assert_eq!(timeouts[self_pubkey], std::u64::MAX);
             assert!(timeouts.contains_key(&Pubkey::default()));
-            rv = CrdsGossipPull::purge_active(thread_pool, &mut self.crds, now, timeouts);
+            rv = CrdsGossipPull::purge_active(thread_pool, &self.crds, now, timeouts);
         }
         self.crds
+            .write()
+            .unwrap()
             .trim_purged(now.saturating_sub(5 * self.pull.crds_timeout));
         self.pull.purge_failed_inserts(now);
         rv
@@ -333,22 +337,24 @@ impl CrdsGossip {
 
     // Only for tests and simulations.
     pub(crate) fn mock_clone(&self) -> Self {
+        let crds = self.crds.read().unwrap().mock_clone();
         Self {
-            crds: self.crds.mock_clone(),
+            crds: RwLock::new(crds),
             push: self.push.mock_clone(),
             pull: self.pull.mock_clone(),
         }
     }
 }
 
-/// Computes a normalized(log of actual stake) stake
+/// Computes a normalized (log of actual stake) stake.
 pub fn get_stake<S: std::hash::BuildHasher>(id: &Pubkey, stakes: &HashMap<Pubkey, u64, S>) -> f32 {
     // cap the max balance to u32 max (it should be plenty)
     let bal = f64::from(u32::max_value()).min(*stakes.get(id).unwrap_or(&0) as f64);
     1_f32.max((bal as f32).ln())
 }
 
-/// Computes bounded weight given some max, a time since last selected, and a stake value
+/// Computes bounded weight given some max, a time since last selected, and a stake value.
+///
 /// The minimum stake is 1 and not 0 to allow 'time since last' picked to factor in.
 pub fn get_weight(max_weight: f32, time_since_last_selected: u32, stake: f32) -> f32 {
     let mut weight = time_since_last_selected as f32 * stake;
@@ -368,12 +374,14 @@ mod test {
 
     #[test]
     fn test_prune_errors() {
-        let mut crds_gossip = CrdsGossip::default();
+        let crds_gossip = CrdsGossip::default();
         let id = Pubkey::new(&[0; 32]);
         let ci = ContactInfo::new_localhost(&Pubkey::new(&[1; 32]), 0);
         let prune_pubkey = Pubkey::new(&[2; 32]);
         crds_gossip
             .crds
+            .write()
+            .unwrap()
             .insert(
                 CrdsValue::new_unsigned(CrdsData::ContactInfo(ci.clone())),
                 0,
